package com.starbuds.server.service.core.redis.listener;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.cache.RedisCacheKeys;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.mq.QueueGiftComboTimeoutMsg;
import com.starbuds.server.service.core.redis.RedisUtil;
import com.starbuds.server.service.core.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.connection.Message;
import org.springframework.data.redis.connection.MessageListener;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.Set;

/**
 * 备注
 *
 * @author Lay
 * @date 2019-04-02
 */
@Component
@Slf4j
public class RedisKeyExpireMsgListener implements MessageListener {

    @Resource
    RedisUtil redisUtil;

    @Resource
    RocketProducer rocketProducer;

    @Override
    public void onMessage(Message message, byte[] bytes) {
        String key = new String(message.getBody());
        if (key.contains(RedisCacheKeys.LIVE_GIFT_COMBO)) {
            String comboKey = key.replace(RedisCacheKeys.LIVE_GIFT_COMBO, "");

            // 连击数处理
            Set<String> keys = redisUtil.keys(RedisCacheKeys.getLiveComboHisPattern(key));
            if (keys.size() == 0) {
                return;
            }

            Long timestamp = 0L;
            for (String hKey : keys) {
                long time = Convert.toLong(hKey.substring(hKey.lastIndexOf("_") + 1));
                if (time > timestamp) {
                    timestamp = time;
                }
            }

            String hisKey = RedisCacheKeys.getLiveComboHis(key, timestamp);
            Long combo = Convert.toLong(redisUtil.getAndSet(hisKey, "-1"), -1L);
            if (combo < 0) {
                return;
            }

            // 解析 key 主播id:用户id:礼物id:数量
            String[] params = comboKey.split("_");
            // 丢出礼物连击过期消息
            QueueGiftComboTimeoutMsg msg = new QueueGiftComboTimeoutMsg();
            msg.setReceiverUserId(Convert.toLong(params[0]));
            msg.setSenderUserId(Convert.toLong(params[1]));
            msg.setGiftId(Convert.toLong(params[2]));
            msg.setQuantity(Convert.toInt(params[3]));
            msg.setCombo(combo);
            rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.GIFT_COMBO_TIMEOUT, msg.serialize());

            redisUtil.delete(Arrays.asList(key, hisKey));
        }
    }
}
